@@ -1,7 +1,19 @@
 import argparse
 import sys
 from argparse import ArgumentParser as ArgparseParser
-from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, TypeVar, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    get_type_hints,
+)
 
 from typing_extensions import assert_never
 
@@ -29,7 +41,7 @@ class Binding:
         if not hasattr(func, "__annotations__"):
             raise ValueError(f"Function {func.__name__} misses type annotations.")
 
-        annotations = func.__annotations__
+        annotations = get_type_hints(func)
 
         if len(annotations) == 0:
             raise ValueError(f"Type annotations of {func.__name__} are empty.")
@@ -63,11 +75,13 @@ class SubParser:
         self,
         name: str,
         args_or_group: "ArgsOrGroup",
+        help: Optional[str] = None,
         aliases: Optional[List[str]] = None,
     ):
         self._name = name
         self._args_or_group = args_or_group
         self._aliases = aliases
+        self._help = help
 
     def __str__(self) -> str:
         return f"SubParser('{self._name}', {_to_string(self._args_or_group)})"
@@ -81,11 +95,13 @@ class SubParserGroup:
         self,
         *subparsers: SubParser,
         common_args: Optional[Type[TypedArgs]] = None,
+        description: Optional[str] = None,
         required: bool = True,
     ):
         self._sub_parser_declarations = subparsers
         self._common_args = common_args
         self._required = required
+        self._description = description
 
     def __str__(self) -> str:
         return f"SubParserGroup({', '.join(map(str, self._sub_parser_declarations))})"
@@ -291,16 +307,23 @@ def _traverse_build_parser(
             argparse_subparsers = parser.add_subparsers(
                 help="Available sub commands",
                 dest=dest,
+                description=group._description,
             )
         else:
             argparse_subparsers = parser.add_subparsers(
                 help="Available sub commands",
                 dest=dest,
+                description=group._description,
                 required=group._required,
             )
 
         for sub_parser_declaration in group._sub_parser_declarations:
-            argparse_subparser = argparse_subparsers.add_parser(sub_parser_declaration._name)
+            kwargs = {}
+            if sub_parser_declaration._help is not None:
+                kwargs["help"] = sub_parser_declaration._help
+            argparse_subparser = argparse_subparsers.add_parser(
+                sub_parser_declaration._name, **kwargs  # type: ignore
+            )
 
             _traverse_build_parser(
                 sub_parser_declaration._args_or_group,
@@ -425,12 +448,25 @@ def _build_add_argument_args(
         "help": _generate_help_text(arg),
     }
 
+    # Unwrap optionals
+    underlying_if_optional = annotation.get_underlying_if_optional()
+    if underlying_if_optional is not None:
+        is_optional = True
+        annotation = underlying_if_optional
+    else:
+        is_optional = False
+
     # Unwrap collections
     underlying_if_list = annotation.get_underlying_if_list()
     if underlying_if_list is not None:
         is_collection = True
         annotation = underlying_if_list
         # TODO: How should we handle e.g. List[bool]?
+
+        # Sanity checks:
+        if arg.nargs_with_default() == "+" and is_optional:
+            raise AssertionError("An argument with nargs='+' must not be optional")
+
     else:
         is_collection = False
 
@@ -438,7 +474,7 @@ def _build_add_argument_args(
     if annotation.is_bool:
         is_required = False
     else:
-        if annotation.is_optional or arg.has_default():
+        if is_optional or arg.has_default():
             is_required = False
         else:
             is_required = True
@@ -493,10 +529,7 @@ def _build_add_argument_args(
 
     # Nargs handling
     if is_collection:
-        if arg.nargs is None:
-            kwargs["nargs"] = "*"
-        else:
-            kwargs["nargs"] = arg.nargs
+        kwargs["nargs"] = arg.nargs_with_default()
 
     # Name handling
     cli_arg_name = python_arg_name.replace("_", "-")
